@@ -33,9 +33,12 @@ that the Maven specific "~/.m2/settings.xml" file is NOT parsed for settings.
 
 #| 
 Test:
-(resolve-dependencies "org.slf4j" "slf4j-api" "1.6.1")
+(abcl-asdf:resolve "org.slf4j:slf4j-api:1.6.1")
+
+(abcl-asdf:resolve "org.apache.maven:maven-aether-provider:3.0.4")
+
+(abcl-asdf:resolve "com.google.gwt:gwt-user")
 
-(resolve-dependencies "org.apache.maven" "maven-aether-provider" "3.0.4")
 |#
 
 (defparameter *maven-verbose* t
@@ -52,13 +55,15 @@ Test:
 
 Returns the path of the Maven executable or nil if none are found.
 
+Returns the version of Maven found as the second value.
+
 Emits warnings if not able to find a suitable executable."
 
   (let ((m2-home (ext:getenv "M2_HOME"))
         (m2 (ext:getenv "M2"))
         (mvn-executable (if (find :unix *features*)
-                               "mvn"
-                               "mvn.bat")))
+                            "mvn"
+                            "mvn.bat")))
     (when (and m2-home (probe-file m2-home))
       (let* ((m2-home (truename m2-home))
              (mvn-path (merge-pathnames 
@@ -66,7 +71,8 @@ Emits warnings if not able to find a suitable executable."
                         m2-home))
              (mvn (truename mvn-path)))
         (if mvn
-            (return-from find-mvn mvn)
+            (values (return-from find-mvn mvn)
+                    (ensure-mvn-version))
             (warn "M2_HOME was set to '~A' in the process environment but '~A' doesn't exist." 
                   m2-home mvn-path))))
     (when (and m2 (probe-file m2))
@@ -74,7 +80,8 @@ Emits warnings if not able to find a suitable executable."
              (mvn-path (merge-pathnames mvn-executable m2))
              (mvn (truename mvn-path)))
         (if mvn
-            (return-from find-mvn mvn)
+            (values (return-from find-mvn mvn)
+                    (ensure-mvn-version))
             (warn "M2 was set to '~A' in the process environment but '~A' doesn't exist." 
                   m2 mvn-path))))
     (let* ((which-cmd 
@@ -84,14 +91,14 @@ Emits warnings if not able to find a suitable executable."
                 "where.exe"))
            (which-cmd-p 
             (handler-case 
-                (sys::run-program which-cmd nil)
+                (sys:run-program which-cmd nil)
               (t () nil))))
       (when which-cmd-p
         (dolist (mvn-path *mavens*)
           (let ((mvn 
                  (handler-case 
-                     (truename (read-line (sys::process-output 
-                                           (sys::run-program 
+                     (truename (read-line (sys:process-output 
+                                           (sys:run-program 
                                             which-cmd `(,mvn-path))))) 
                    (end-of-file () nil)
                    (t (e) 
@@ -100,41 +107,60 @@ Emits warnings if not able to find a suitable executable."
                              mvn-path e)))))
             (when mvn
               (return-from find-mvn mvn)))))))
-  (warn "Unable to locate Maven executable."))
+  (warn "Unable to locate Maven executable to find Maven Aether adaptors."))
 
 (defun find-mvn-libs ()
-  (let ((mvn (find-mvn)))
-    (unless mvn
-      (warn "Failed to find Maven3 libraries.")
-      (return-from find-mvn-libs nil))
-    (truename (make-pathname 
-               :defaults (merge-pathnames "../lib/" mvn)
-               :name nil :type nil))))
+  (unless (find-mvn)
+    (warn "Failed to find Maven executable to determine Aether library location."))
+  (some 
+   (lambda (d)
+     (when (directory (merge-pathnames "maven-core-*.jar" d))
+       (truename d)))
+   (list (make-pathname :defaults (merge-pathnames "../lib/" (find-mvn))
+                        :name nil :type nil)
+         ;; library location for homebrew maven package on OS X
+         (make-pathname :defaults (merge-pathnames "../libexec/lib/" (find-mvn))
+                        :name nil :type nil)
+         #p"/usr/local/share/java/maven3/lib/" ;; FreeBSD ports
+         #p"/usr/local/maven/lib/"))) ;; OpenBSD location suggested by Timo Myyrä
 
 (defparameter *mvn-libs-directory*
   nil
   "Location of 'maven-core-3.<m>.<p>.jar', 'maven-embedder-3.<m>.<p>.jar' etc.")
 
 (defun mvn-version ()
-  "Return the Maven version used by the Aether connector."
-  (let ((stream (sys::process-output
-                 (sys::run-program (truename (find-mvn)) '("-version"))))
-        (pattern (#"compile"
-                  'regex.Pattern
-                  "Apache Maven ([0-9]+)\\.([0-9]+)\\.([0-9]+)")))
-    (do ((line (read-line stream nil :eof) 
-              (read-line stream nil :eof)))
-        ((or (not line) (eq line :eof)) nil)
-      (let ((matcher (#"matcher" pattern line)))
-        (when (#"find" matcher)
-          (return-from mvn-version
-            (handler-case 
+  "Return the Maven version used by the Aether connector located by
+  FIND-MVN as a list of (MAJOR MINOR PATHLEVEL) integers.
+
+Signals a simple-error with additional information if this attempt fails."
+  (handler-case 
+      (let* ((process (sys:run-program (truename (find-mvn)) '("-version")))
+             (output (sys:process-output process))
+             (pattern (#"compile"
+                       'regex.Pattern
+                       "Apache Maven ([0-9]+)\\.([0-9]+)\\.([0-9]+)"))
+             lines)
+        (do ((line (read-line output nil :eof) 
+                   (read-line output nil :eof)))
+            ((or (not line) (eq line :eof)) nil)
+          (push line lines)
+          (let ((matcher (#"matcher" pattern line)))
+            (when (#"find" matcher)
+              (return-from mvn-version
                 (mapcar #'parse-integer 
                         `(,(#"group" matcher 1) 
                            ,(#"group" matcher 2) 
-                           ,(#"group" matcher 3)))
-              (t (e) 
-                (error "Failed to parse Maven version from ~A because~&~A." line e)))))))))
+                           ,(#"group" matcher 3)))))))
+        (when lines 
+          (signal "No parseable Maven version found in ~{~&  ~A~}" (nreverse lines)))
+        (let ((error (sys:process-error process)))
+          (do ((line (read-line error nil :eof) 
+                     (read-line error nil :eof)))
+              ((or (not line) (eq line :eof)) nil)
+            (push line lines)
+            (signal "Invocation of Maven returned the error ~{~&  ~A~}" (nreverse lines)))))
+    (t (e) 
+      (error "Failed to determine Maven version: ~A." e))))
 
 (defun ensure-mvn-version ()
   "Return t if Maven version is 3.0.3 or greater."
@@ -142,25 +168,34 @@ Emits warnings if not able to find a suitable executable."
          (major (first version))
          (minor (second version))
          (patch (third version)))
-    (or 
-     (and (>= major 3)
-          (>= minor 1))
-     (and (>= major 3)
-          (>= minor 0)
-          (>= patch 3)))))
+    (values
+     (or 
+      (and (>= major 3)
+           (>= minor 1))
+      (and (>= major 3)
+           (>= minor 0)
+           (>= patch 3)))
+     (list major minor patch))))
 
 (defparameter *init* nil)
 
 (defun init (&optional &key (force nil))
- "Run the initialization strategy to bootstrap a Maven dependency node."
- (unless (or force *mvn-libs-directory*)
-   (setf *mvn-libs-directory* (find-mvn-libs)))
+  "Run the initialization strategy to bootstrap a Maven dependency node.
+
+Set *MVN-LIBS-DIRECTORY* to an explicit value before running this
+function in order to bypass the dynamic introspection of the location
+of the mvn executable with an explicit value."
+  (when force
+    (setf *session* nil
+          *repository-system* nil))
+  (unless (or force *mvn-libs-directory*)
+    (setf *mvn-libs-directory* (find-mvn-libs)))
   (unless (and *mvn-libs-directory*
                (probe-file *mvn-libs-directory*))
-   (error "Please obtain and install maven-3.0.4 locally from http://maven.apache.org/download.html, then set ABCL-ASDF:*MVN-DIRECTORY* appropiately."))
- (unless (ensure-mvn-version)
-   (error "We need maven-3.0.3 or later."))  (add-directory-jars-to-class-path *mvn-libs-directory* nil)
-  (setf *init* t))
+    (error "Please obtain and install maven-3.0.3 or later locally from <http://maven.apache.org/download.html>, then set ABCL-ASDF:*MVN-LIBS-DIRECTORY* to the directory containing maven-core-3.*.jar et. al."))
+  (unless (ensure-mvn-version)
+    (error "We need maven-3.0.3 or later."))  (add-directory-jars-to-class-path *mvn-libs-directory* nil)
+    (setf *init* t))
 
 (defun find-http-wagon ()
   "Find an implementation of the object that provides access to http and https resources.
@@ -178,10 +213,18 @@ maso2000 in the Manual.)"
   "Returns an implementation of the org.sonatype.aether.connector.wagon.WagonProvider contract.
 
 The implementation is specified as Lisp closures.  Currently, it only
-specializes the lookup() method if passed an 'http' role hint."
+specializes the lookup() method if passed an 'http' or an 'https' role
+hint."
   (unless *init* (init))
   (java:jinterface-implementation 
-   "org.sonatype.aether.connector.wagon.WagonProvider"
+   (#"getName" 
+    (or
+     (ignore-errors  ;; Maven 3.2.5+
+       (jss:find-java-class 'aether.transport.wagon.WagonProvider))
+     (ignore-errors  ;; Maven 3.1.0+
+       (jss:find-java-class 'aether.connector.wagon.WagonProvider))
+     (ignore-errors  ;; Maven 3.0.x
+       (jss:find-java-class 'org.sonatype.aether.connector.wagon.WagonProvider))))
    "lookup"
    (lambda (role-hint)
      (cond 
@@ -191,58 +234,104 @@ specializes the lookup() method if passed an 'http' role hint."
         (progn 
           (format *maven-verbose* 
                   "~&WagonProvider stub passed '~A' as a hint it couldn't satisfy.~%" role-hint)
-           java:+null+))))
+          java:+null+))))
    "release"
    (lambda (wagon)
      (declare (ignore wagon)))))
 
 (defun find-service-locator ()
-  (handler-case 
-      (java:jnew "org.apache.maven.repository.internal.MavenServiceLocator") ;; maven-3.0.4
-    (error () 
-      (java:jnew "org.apache.maven.repository.internal.DefaultServiceLocator"))))
+  (or 
+   (ignore-errors 
+     (#"newServiceLocator" 'org.apache.maven.repository.internal.MavenRepositorySystemUtils)) ;; maven-3.1.0
+   (ignore-errors
+     (java:jnew "org.apache.maven.repository.internal.MavenServiceLocator")) ;; maven-3.0.4
+   (ignore-errors
+     (java:jnew "org.apache.maven.repository.internal.DefaultServiceLocator"))
+   (ignore-errors  ;; maven-3.1.0 using org.eclipse.aether...
+     (jss:find-java-class 'aether.impl.DefaultServiceLocator))))
 
 (defun make-repository-system ()
   (unless *init* (init))
   (let ((locator 
          (find-service-locator))
         (wagon-provider-class 
-         (java:jclass "org.sonatype.aether.connector.wagon.WagonProvider"))
+         (or
+          (ignore-errors ;; Maven-3.3.x
+            (jss:find-java-class 'connector.transport.TransporterFactory))
+          (ignore-errors ;; Maven-3.2.5
+            (jss:find-java-class 'org.eclipse.aether.transport.wagon.WagonProvider))
+          (ignore-errors  ;; Maven-3.1.x 
+            (jss:find-java-class 'aether.connector.wagon.WagonProvider))
+          (ignore-errors 
+            (java:jclass "org.sonatype.aether.connector.wagon.WagonProvider"))))
         (wagon-repository-connector-factory-class
-         (java:jclass "org.sonatype.aether.connector.wagon.WagonRepositoryConnectorFactory"))
+         (or 
+          (ignore-errors 
+            (jss:find-java-class 'org.eclipse.aether.connector.basic.BasicRepositoryConnectorFactory))
+          (ignore-errors 
+            (jss:find-java-class 'aether.connector.wagon.WagonRepositoryConnectorFactory))
+          (ignore-errors 
+            (java:jclass "org.sonatype.aether.connector.wagon.WagonRepositoryConnectorFactory"))))
         (repository-connector-factory-class 
-         (java:jclass "org.sonatype.aether.spi.connector.RepositoryConnectorFactory"))
+         (or
+          (ignore-errors
+            (jss:find-java-class 'org.eclipse.aether.spi.connector.RepositoryConnectorFactory))
+          (ignore-errors
+            (jss:find-java-class 'aether.spi.connector.RepositoryConnectorFactory))
+          (ignore-errors 
+            (java:jclass "org.sonatype.aether.spi.connector.RepositoryConnectorFactory"))))
         (repository-system-class
-         (java:jclass "org.sonatype.aether.RepositorySystem")))
-    (#"setServices" locator
-                    wagon-provider-class
-                   (java:jarray-from-list
-                    (list (make-wagon-provider))))
+         (or
+          (ignore-errors 
+            (jss:find-java-class 'org.eclipse.aether.RepositorySystem))
+          (ignore-errors 
+            (jss:find-java-class 'aether.RepositorySystem))
+          (ignore-errors
+            (java:jclass "org.sonatype.aether.RepositorySystem")))))
+
+    (if (equal wagon-provider-class (ignore-errors (jss:find-java-class 'TransporterFactory)))
+        ;;; Maven-3.3.3
+        (let ((wagon-transporter-factory (jss:new 'WagonTransporterFactory)))
+          (#"setWagonProvider" wagon-transporter-factory (make-wagon-provider))
+          (#"setServices" locator
+                          wagon-provider-class
+                          (java:jarray-from-list (list wagon-transporter-factory))))
+        (#"setServices" locator
+                        wagon-provider-class
+                        (java:jarray-from-list
+                         (list (make-wagon-provider)))))
     (#"addService" locator
                    repository-connector-factory-class
                    wagon-repository-connector-factory-class)
     (values (#"getService" locator
                            repository-system-class)
             locator)))
-        
+
 (defun make-session (repository-system)
-  "Construct a new org.sonatype.aether.RepositorySystemSession from REPOSITORY-SYSTEM"
-  (let ((session 
-         (java:jnew (jss:find-java-class "MavenRepositorySystemSession")))
+  "Construct a new aether.RepositorySystemSession from the specified REPOSITORY-SYSTEM."
+  (let ((session
+         (or 
+          (ignore-errors (#"newSession" 'org.apache.maven.repository.internal.MavenRepositorySystemUtils))
+          (ignore-errors (java:jnew (jss:find-java-class "MavenRepositorySystemSession")))))
         (local-repository 
          (java:jnew (jss:find-java-class "LocalRepository")
-                  (namestring (merge-pathnames ".m2/repository/"
-                                               (user-homedir-pathname))))))
+                    (namestring (merge-pathnames ".m2/repository/"
+                                                 (user-homedir-pathname))))))
     (#"setLocalRepositoryManager" 
      session
-     (#"newLocalRepositoryManager" repository-system
-                                   local-repository))))
+     (or 
+      (ignore-errors      ;; maven-3.1.0
+        (#"newLocalRepositoryManager" 
+         repository-system session local-repository))
+      (ignore-errors 
+        (#"newLocalRepositoryManager" 
+         repository-system local-repository))))))
 
 (defparameter *maven-http-proxy* nil
   "A string containing the URI of an http proxy for Maven to use.")
 
 (defun make-proxy ()
-  "Return an org.sonatype.aether.repository.Proxy instance initialized from *MAVEN-HTTP-PROXY*."
+  "Return an aether.repository.Proxy instance initialized from *MAVEN-HTTP-PROXY*."
   (unless *maven-http-proxy*
     (warn "No proxy specified in *MAVEN-HTTP-PROXY*")
     (return-from make-proxy nil))
@@ -256,11 +345,16 @@ specializes the lookup() method if passed an 'http' role hint."
                  (parse-integer (subseq authority (1+ (search ":" authority))))))
          ;; TODO allow specification of authentication
          (authentication java:+null+))
-    (jss:new 'org.sonatype.aether.repository.Proxy
-             scheme host port authentication)))
+    (or 
+     (ignore-errors
+       (jss:new 'org.eclipse.aether.repository.Proxy
+                scheme host port authentication))
+     (ignore-errors
+       (jss:new 'org.sonatype.aether.repository.Proxy
+                scheme host port authentication)))))
 
 (defparameter *repository-system*  nil
-  "The org.sonatype.aether.RepositorySystem used by the Maeven Aether connector.")
+  "The aether.RepositorySystem used by the Maeven Aether connector.")
 (defun ensure-repository-system (&key (force nil))
   (when (or force (not *repository-system*))
     (setf *repository-system* (make-repository-system)))
@@ -282,10 +376,26 @@ If *MAVEN-HTTP-PROXY* is non-nil, parse its value as the http proxy."
                 proxy 
                 ;; A string specifying non proxy hosts, or null
                 java:+null+))))
-    *session*)
+  *session*)
+
+(defun make-artifact (artifact-string)
+  "Return an instance of aether.artifact.DefaultArtifact initialized from ARTIFACT-STRING." 
+  (or
+   (ignore-errors
+     (jss:new 'aether.artifact.DefaultArtifact artifact-string))
+   (ignore-errors
+     (jss:new "org.sonatype.aether.util.artifact.DefaultArtifact" artifact-string))))
+
+(defun make-artifact-request () 
+  "Construct a new aether.resolution.ArtifactRequest."
+  (or 
+   (ignore-errors
+     (java:jnew (jss:find-java-class 'aether.resolution.ArtifactRequest)))
+   (ignore-errors
+     (java:jnew "org.sonatype.aether.resolution.ArtifactRequest"))))
 
 ;;; TODO change this to work on artifact strings like log4j:log4j:jar:1.2.16
-(defun resolve-artifact (group-id artifact-id &optional (version "LATEST" versionp))
+(defun resolve-artifact (group-id artifact-id &key (version "LATEST" versionp))
   "Resolve artifact to location on the local filesystem.
 
 Declared dependencies are not attempted to be located.
@@ -296,11 +406,12 @@ Returns the Maven specific string for the artifact "
   (unless versionp
     (warn "Using LATEST for unspecified version."))
   (unless *init* (init))
-  (let* ((artifact-string (format nil "~A:~A:~A" group-id artifact-id version))
+  (let* ((artifact-string 
+          (format nil "~A:~A:~A" group-id artifact-id version))
          (artifact 
-          (jss:new "org.sonatype.aether.util.artifact.DefaultArtifact" artifact-string))
+          (make-artifact artifact-string))
          (artifact-request 
-          (java:jnew "org.sonatype.aether.resolution.ArtifactRequest")))
+          (make-artifact-request)))
     (#"setArtifact" artifact-request artifact)
     (#"addRepository" artifact-request (ensure-remote-repository))
     (#"toString" (#"getFile" 
@@ -308,16 +419,20 @@ Returns the Maven specific string for the artifact "
                                                       (ensure-session) artifact-request))))))
 
 (defun make-remote-repository (id type url) 
-  (jss:new 'aether.repository.RemoteRepository id type url))
+  (or 
+   (ignore-errors 
+     (#"build" (jss:new "org.eclipse.aether.repository.RemoteRepository$Builder" id type url)))
+   (ignore-errors
+     (jss:new 'aether.repository.RemoteRepository id type url))))
 
 (defparameter *default-repository* 
-   "http://repo1.maven.org/maven2/")
+  "http://repo1.maven.org/maven2/")
 
 (defun add-repository (repository)
   (ensure-remote-repository :repository repository))
 
 (defparameter *maven-remote-repository*  nil
-    "The remote repository used by the Maven Aether embedder.")
+  "The remote repository used by the Maven Aether embedder.")
 (defun ensure-remote-repository (&key 
                                    (force nil)
                                    (repository *default-repository* repository-p))
@@ -331,12 +446,13 @@ Returns the Maven specific string for the artifact "
       (setf *maven-remote-repository* r)))
   *maven-remote-repository*)
 
+
 (defun resolve-dependencies (group-id artifact-id 
-                             &optional  ;;; XXX Uggh.  Move to keywords when we get the moxie.
-                             (version "LATEST" versionp)
-                             (repository *maven-remote-repository* repository-p))
+                             &key
+                               (version "LATEST" versionp)
+                               (repository *maven-remote-repository* repository-p))
   "Dynamically resolve Maven dependencies for item with GROUP-ID and ARTIFACT-ID 
-optionally with a VERSION and a REPOSITORY.  Users of the function are advised 
+optionally with a VERSION and a REPOSITORY.  
 
 All recursive dependencies will be visited before resolution is successful.
 
@@ -347,26 +463,32 @@ in Java CLASSPATH representation."
   (unless *init* (init))
   (unless versionp
     (warn "Using LATEST for unspecified version."))
-  (let* ((artifact
-          (java:jnew (jss:find-java-class "aether.util.artifact.DefaultArtifact")
-                     (format nil "~A:~A:~A"
-                             group-id artifact-id version)))
+  (let* ((coords 
+          (format nil "~A:~A:~A" group-id artifact-id (if versionp version "LATEST")))
+         (artifact 
+          (make-artifact coords))
          (dependency 
-          (java:jnew (jss:find-java-class "aether.graph.Dependency")
-                     artifact (java:jfield (jss:find-java-class "JavaScopes") "RUNTIME")))
+          (java:jnew (jss:find-java-class 'aether.graph.Dependency)
+                     artifact (java:jfield (jss:find-java-class "JavaScopes") "COMPILE")))
          (collect-request (java:jnew (jss:find-java-class "CollectRequest"))))
     (#"setRoot" collect-request dependency)
-    (#"addRepository" collect-request 
-                      (if repository-p
-                          (ensure-remote-repository :repository repository)
-                          (ensure-remote-repository)))
+     ;; Don't call addRepository if we explicitly specify a NIL repository
+    (unless (and repository-p (not repository))
+      (#"addRepository" collect-request 
+                        (if repository-p
+                            (ensure-remote-repository :repository repository)
+                            (ensure-remote-repository))))
     (let* ((node 
             (#"getRoot" (#"collectDependencies" (ensure-repository-system) (ensure-session) collect-request)))
-           (dependency-request 
+           (dependency-request
+            ;;; pre Maven-3.3.x
+            #+nil
             (java:jnew (jss:find-java-class "DependencyRequest")
-                       node java:+null+))
+                       node java:+null+)
+            (jss:new 'DependencyRequest))
            (nlg 
             (java:jnew (jss:find-java-class "PreorderNodeListGenerator"))))
+      (#"setRoot" dependency-request node)
       (#"resolveDependencies" (ensure-repository-system) (ensure-session) dependency-request)
       (#"accept" node nlg)
       (#"getClassPath" nlg))))
@@ -375,7 +497,7 @@ in Java CLASSPATH representation."
   (flet ((log (e) 
            (format *maven-verbose* "~&~A~%" (#"toString" e))))
     (java:jinterface-implementation 
-     "org.sonatype.aether.RepositoryListener"
+     (#"getName" (jss:find-java-class 'aether.RepositoryListener))
      "artifactDeployed" 
      #'log
      "artifactDeploying" 
@@ -415,7 +537,7 @@ in Java CLASSPATH representation."
      "metadataResolving"
      #'log)))
 
-         
+
 (defmethod resolve ((string string))
   "Resolve a colon separated GROUP-ID:ARTIFACT-ID[:VERSION] reference to a Maven artifact.
 
@@ -429,12 +551,22 @@ artifact and all of its transitive dependencies."
     (cond 
       ((= (length result) 3)
        (resolve-dependencies 
-        (first result) (second result) (third result)))
+        (first result) (second result) :version (third result)))
       ((string= string "com.sun.jna:jna")
        (warn "Replacing request for no longer available com.sun.jna:jna with net.java.dev.jna:jna")
-       (resolve-dependencies "net.java.dev.jna" "jna" "LATEST"))
-      (t
-       (apply #'resolve-dependencies result)))))
-  
+       (resolve-dependencies "net.java.dev.jna" "jna" :version "LATEST"))
+      ((= (length result) 2)
+       (resolve-dependencies
+        (first result) (second result)))
+      (t 
+       (destructuring-bind (group-id artifact-id &optional version repository)
+           (split-string string "/")
+         (setf result 
+               (apply #'resolve-dependencies group-id artifact-id
+                      (append (when version
+                                `(:version ,version))
+                              (when repository
+                                `(:repository ,repository))))))))))
+
 ;;; Currently the last file listed in ASDF
 (provide 'abcl-asdf)
