@@ -213,12 +213,8 @@ public final class Bignum extends LispInteger
   @Override
   public boolean equalp(LispObject obj)
   {
-    if (obj instanceof Bignum)
-      return value.equals(((Bignum)obj).value);
-    if (obj instanceof SingleFloat)
-      return floatValue() == ((SingleFloat)obj).value;
-    if (obj instanceof DoubleFloat)
-      return doubleValue() == ((DoubleFloat)obj).value;
+    if (obj != null && obj.numberp())
+      return isEqualTo(obj);
     return false;
   }
 
