@@ -2,7 +2,7 @@
  * LispObject.java
  *
  * Copyright (C) 2002-2007 Peter Graves
- * $Id: LispObject.java 13750 2012-01-10 20:11:27Z mevenson $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -670,10 +670,8 @@ public class LispObject //extends Lisp
                     SimpleString doc = new SimpleString(docstring);
                     ((Symbol)this).setDocumentation(Symbol.FUNCTION, doc);
                     return doc;
-                } else if (fn instanceof StandardGenericFunction) {
-                    return 
-                        StandardGenericFunction.checkStandardGenericFunction(fn)
-                        .slots[StandardGenericFunctionClass.SLOT_INDEX_DOCUMENTATION];
+                } else if (fn.typep(Symbol.STANDARD_GENERIC_FUNCTION) != NIL) {
+                    return Symbol.SLOT_VALUE.execute(fn, Symbol._DOCUMENTATION);
                 }
             }
         }
